@@ -58,6 +58,7 @@ struct _ECalendarViewPrivate {
 	GtkTargetList *paste_target_list;
 
 	gboolean allow_direct_summary_edit;
+	gboolean allow_event_dnd;
 };
 
 enum {
@@ -67,7 +68,8 @@ enum {
 	PROP_PASTE_TARGET_LIST,
 	PROP_TIME_DIVISIONS,
 	PROP_IS_EDITING,
-	PROP_ALLOW_DIRECT_SUMMARY_EDIT
+	PROP_ALLOW_DIRECT_SUMMARY_EDIT,
+	PROP_ALLOW_EVENT_DND
 };
 
 /* FIXME Why are we emitting these event signals here? Can't the model just be listened to? */
@@ -126,20 +128,21 @@ calendar_view_delete_event (ECalendarView *cal_view,
 {
 	ECalModel *model;
 	ECalComponent *comp;
-	ECalComponentVType vtype;
 	ESourceRegistry *registry;
 	ECalClient *client;
 	ICalComponent *icalcomp;
 	ICalTime *itt_start = NULL, *itt_end = NULL;
 	time_t instance_start;
 	gboolean do_delete = TRUE;
+	gboolean organizer_is_user;
+	gboolean attendee_is_user;
+	gboolean can_send_notice = FALSE;
 
 	model = e_calendar_view_get_model (cal_view);
 	registry = e_cal_model_get_registry (model);
 
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, i_cal_component_clone (sel_data->icalcomp));
-	vtype = e_cal_component_get_vtype (comp);
 
 	cal_comp_get_instance_times (sel_data->client, sel_data->icalcomp,
 		e_cal_model_get_timezone (model),
@@ -158,25 +161,29 @@ calendar_view_delete_event (ECalendarView *cal_view,
 	if (!only_occurrence && !e_cal_client_check_recurrences_no_master (client))
 		e_cal_component_set_recurid (comp, NULL);
 
+	organizer_is_user = itip_organizer_is_user (registry, comp, client);
+	attendee_is_user = itip_attendee_is_user (registry, comp, client);
+
 	if (e_cal_model_get_confirm_delete (model)) {
-		do_delete = e_cal_dialogs_delete_component (
-			comp, FALSE, 1, vtype, GTK_WIDGET (cal_view));
+		GtkWindow *parent_window;
+		GtkWidget *toplevel;
+
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (cal_view));
+		parent_window = GTK_IS_WINDOW (toplevel) ? GTK_WINDOW (toplevel) : NULL;
+
+		do_delete = e_cal_dialogs_delete_with_comment (parent_window, client, comp, organizer_is_user, attendee_is_user, &can_send_notice);
 	}
 
 	if (do_delete) {
 		ECalOperationFlags op_flags = E_CAL_OPERATION_FLAG_NONE;
-		GtkWindow *parent_window;
-		gboolean organizer_is_user;
 		const gchar *uid;
 		gchar *rid;
 
 		rid = e_cal_component_get_recurid_as_string (comp);
-		organizer_is_user = itip_organizer_is_user (registry, comp, client);
-		parent_window = (GtkWindow *) gtk_widget_get_toplevel (GTK_WIDGET (cal_view));
 
 		if (itip_has_any_attendees (comp) && (organizer_is_user ||
 		    itip_sentby_is_user (registry, comp, client))) {
-			if (e_cal_dialogs_cancel_component (parent_window, client, comp, FALSE, organizer_is_user)) {
+			if (can_send_notice) {
 				if (only_occurrence && !e_cal_component_is_instance (comp)) {
 					ECalComponentRange *range;
 					ECalComponentDateTime *dtstart;
@@ -208,10 +215,16 @@ calendar_view_delete_event (ECalendarView *cal_view,
 			} else {
 				op_flags = E_CAL_OPERATION_FLAG_DISABLE_ITIP_MESSAGE;
 			}
-		} else if (e_cal_client_check_save_schedules (client) &&
-			   itip_attendee_is_user (registry, comp, client) &&
-			   !e_cal_dialogs_cancel_component (parent_window, client, comp, FALSE, organizer_is_user)) {
-			op_flags = E_CAL_OPERATION_FLAG_DISABLE_ITIP_MESSAGE;
+		} else if (attendee_is_user) {
+			if (can_send_notice && !e_cal_client_check_save_schedules (client)) {
+				itip_send_component_with_model (model, I_CAL_METHOD_CANCEL,
+					comp, client, NULL, NULL,
+					NULL, E_ITIP_SEND_COMPONENT_FLAG_STRIP_ALARMS);
+				/* attendees will know by the above cancel message */
+				op_flags = E_CAL_OPERATION_FLAG_DISABLE_ITIP_MESSAGE;
+			} else if (!can_send_notice) {
+				op_flags = E_CAL_OPERATION_FLAG_DISABLE_ITIP_MESSAGE;
+			}
 		}
 
 		uid = e_cal_component_get_uid (comp);
@@ -306,6 +319,12 @@ calendar_view_set_property (GObject *object,
 				E_CALENDAR_VIEW (object),
 				g_value_get_boolean (value));
 			return;
+
+		case PROP_ALLOW_EVENT_DND:
+			e_calendar_view_set_allow_event_dnd (
+				E_CALENDAR_VIEW (object),
+				g_value_get_boolean (value));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -349,6 +368,10 @@ calendar_view_get_property (GObject *object,
 		case PROP_ALLOW_DIRECT_SUMMARY_EDIT:
 			g_value_set_boolean (value, e_calendar_view_get_allow_direct_summary_edit (E_CALENDAR_VIEW (object)));
 			return;
+
+		case PROP_ALLOW_EVENT_DND:
+			g_value_set_boolean (value, e_calendar_view_get_allow_event_dnd (E_CALENDAR_VIEW (object)));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -793,7 +816,7 @@ paste_clipboard_data_free (gpointer ptr)
 
 				if (itip_has_any_attendees (comp) && (organizer_is_user ||
 				    itip_sentby_is_user (registry, comp, sel_data->client))) {
-					if (e_cal_dialogs_cancel_component ((GtkWindow *) pcd->top_level, sel_data->client, comp, FALSE, organizer_is_user)) {
+					if (e_cal_dialogs_cancel_component ((GtkWindow *) pcd->top_level, sel_data->client, comp, TRUE, organizer_is_user)) {
 						itip_send_component_with_model (model, I_CAL_METHOD_CANCEL,
 							comp, sel_data->client, NULL, NULL, NULL,
 							E_ITIP_SEND_COMPONENT_FLAG_STRIP_ALARMS | E_ITIP_SEND_COMPONENT_FLAG_ENSURE_MASTER_OBJECT);
@@ -1325,6 +1348,16 @@ e_calendar_view_class_init (ECalendarViewClass *class)
 			FALSE,
 			G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_ALLOW_EVENT_DND,
+		g_param_spec_boolean (
+			"allow-event-dnd",
+			"Whether can drag-and-drop events",
+			NULL,
+			TRUE,
+			G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
 	signals[POPUP_EVENT] = g_signal_new (
 		"popup-event",
 		G_TYPE_FROM_CLASS (class),
@@ -1431,6 +1464,8 @@ e_calendar_view_init (ECalendarView *calendar_view)
 	target_list = gtk_target_list_new (NULL, 0);
 	e_target_list_add_calendar_targets (target_list, 0);
 	calendar_view->priv->paste_target_list = target_list;
+
+	calendar_view->priv->allow_event_dnd = TRUE;
 }
 
 static void
@@ -2113,3 +2148,25 @@ e_calendar_view_set_allow_direct_summary_edit (ECalendarView *cal_view,
 
 	g_object_notify (G_OBJECT (cal_view), "allow-direct-summary-edit");
 }
+
+gboolean
+e_calendar_view_get_allow_event_dnd (ECalendarView *cal_view)
+{
+	g_return_val_if_fail (E_IS_CALENDAR_VIEW (cal_view), FALSE);
+
+	return cal_view->priv->allow_event_dnd;
+}
+
+void
+e_calendar_view_set_allow_event_dnd (ECalendarView *cal_view,
+				     gboolean allow)
+{
+	g_return_if_fail (E_IS_CALENDAR_VIEW (cal_view));
+
+	if ((cal_view->priv->allow_event_dnd ? 1 : 0) == (allow ? 1 : 0))
+		return;
+
+	cal_view->priv->allow_event_dnd = allow;
+
+	g_object_notify (G_OBJECT (cal_view), "allow-event-dnd");
+}
